/**************************************************************
 * clock_module.cpp
 * written by Steve Sprang
 * Copyright  1997
 *
 *************************************************************/
 
#include "module.h"

// some useful global variables
BView		*owner;
BRect		bounds;
BBitmap		*bmap;
BView		*v;
int			pulse_ct = 0;
bool		draw_date;
double		time_clicked;

long module_get_width()
{
	// predetermined size
	return 64;
}

void module_create(BView *mod)
{
	owner = mod; // store the owning view
	bounds = owner->Bounds(); // store its bounds
	
	// create a new bmap for drawing
	bmap = new BBitmap(bounds, B_COLOR_8_BIT, TRUE);
	v = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	bmap->AddChild(v);
}

void module_pulse()
{
	// don't need to do something every pulse so keep a count
	pulse_ct = (pulse_ct + 1) % 3;
	
	// if the count is cool redraw
	if(!pulse_ct) {
		module_draw();
	}
	
	// if we're drawing the date compare the current time to the click timestamp
	// if it's greater than 2 seconds stop drawing the date
	if(draw_date) {
		if((system_time() - time_clicked) / 1000000 > 2) {
			draw_date = false;
		}
	}
}

void module_draw()
{
	short		hours,minutes,seconds, month, day;
	struct tm	*loctime;
	time_t		current;

	// timestuff stolen from the clock source
	current = time(0);
	loctime = localtime(&current);
	hours = loctime->tm_hour;
	minutes = loctime->tm_min;
	seconds = loctime->tm_sec;
	day = loctime->tm_mday;
	month = loctime->tm_mon;
	
	// convert from 24-hour time
	hours = hours % 12;
	if(hours == 0) {
		hours = 12;
	}
	
	bmap->Lock();
	// draw cool 3d background
	v->SetHighColor(kBGCOLOR);
	v->FillRect(bounds);
	
	BPoint tl, tr, bl, br;
	tl = BPoint(bounds.left, bounds.top);
	tr = BPoint(bounds.right, bounds.top);
	bl = BPoint(bounds.left, bounds.bottom);
	br = BPoint(bounds.right, bounds.bottom);
	
	v->SetHighColor(kSHADOW);
	v->StrokeLine(br, tr);
	v->StrokeLine(br, bl);
		
	v->SetHighColor(kHILITE);
	v->StrokeLine(tl, tr);
	v->StrokeLine(tl, bl);
	
	v->SetHighColor(0,0,0);
	v->SetLowColor(kBGCOLOR);
	// stop cool 3d background
	
	// create a time string
	char	time[255], date[255];
	sprintf(time, "%d:%.2d:%.2d", hours, minutes, seconds);
	
	// big nasty switch
	switch(month) {
		case 0: sprintf(date, "Jan-%.2d", day);
				break;
		case 1: sprintf(date, "Feb-%.2d", day);
				break;
		case 2: sprintf(date, "Mar-%.2d", day);
				break;
		case 3:	sprintf(date, "Apr-%.2d", day);
				break;
		case 4:	sprintf(date, "May-%.2d", day);
				break;
		case 5:	sprintf(date, "Jun-%.2d", day);
				break;
		case 6:	sprintf(date, "Jul-%.2d", day);
				break;
		case 7:	sprintf(date, "Aug-%.2d", day);
				break;
		case 8:	sprintf(date, "Sep-%.2d", day);
				break;
		case 9:	sprintf(date, "Oct-%.2d", day);
				break;
		case 10:sprintf(date, "Nov-%.2d", day);
				break;
		case 11:sprintf(date, "Dec-%.2d", day);
				break;
	}
	
	// draw the date if it's set
	if(draw_date) {
		float width = v->StringWidth(date);
		v->DrawString(date, BPoint((bounds.Width() - width)/2, 19)); // centered
	}
	else { // otherwise draw the time
		float width = v->StringWidth(time);
		v->DrawString(time, BPoint((bounds.Width() - width)/2, 19)); // centered
	}
	v->Sync();
	bmap->Unlock();
	
	// blit the bmap
	owner->Window()->Lock();
	owner->DrawBitmap(bmap);
	owner->Window()->Unlock();
}

void module_quit()
{
	// free memory
	delete bmap;
}

void module_message_received(BMessage *msg)
{
	// nothing
}

void module_mouse_down(BPoint, long)
{
	// user wants to see date, record timestamp
	draw_date = true;
	time_clicked = system_time();
	// redraw
	module_draw();
}

void module_mouse_moved(BPoint point, ulong transit, BMessage *msg)
{
	// nothing
}

	
	