/******************************************************************
 * Template for creating PowerBar modules
 * written by Steve Sprang
 * Copyright  1997
 *
 * Note: The functions below are sufficient for simple modules.
 * They are intended to allow user interaction without adding more
 * views to the parent view. It is possible to ignore all the
 * functions except module_get_width(), module_create(), and
 * module_quit(), performing all user interaction by adding custom
 * views to the owning view.
 * 
 ******************************************************************/ 
 
#include "module.h"

/*
these variable are not required, but global variables
are needed to maintain the state of the module. The following
are useful to keep around.
*/

BView		*owner;
BRect		bounds;
BBitmap		*bmap;
BView		*v;

/*
This function is called when the app first opens the module.
It should return the amount of space the module would like to get
*/
long module_get_width()
{
	// make a square
	return kHEIGHT;
}

/*
This is called when the app actually creates a view for the module.
The view is passed as a parameter.
*/
void module_create(BView *mod)
{
	// store the owning view and its bounds
	owner = mod;
	bounds = owner->Bounds();
	
	// create a bmap for drawing
	bmap = new BBitmap(bounds, B_COLOR_8_BIT, TRUE);
	v = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	bmap->AddChild(v);
}

/*
This is called every time a pulse is registered in the owning
view. The PowerBar window is currently set to pulse every 1/3
second or so.
*/
void module_pulse()
{
	// nothing
}

/*
Called when the owning view is destroyed. Clean up allocated
variables here
*/
void module_quit()
{
	// free memory
	delete bmap;
}

/*
Called when the owning view wants the module to redraw.
*/
void module_draw()
{
	bmap->Lock();
	
	/* cool 3d background stuff */
	v->SetHighColor(kBGCOLOR);
	v->FillRect(bounds);
	
	BPoint tl, tr, bl, br;
	tl = BPoint(bounds.left, bounds.top);
	tr = BPoint(bounds.right, bounds.top);
	bl = BPoint(bounds.left, bounds.bottom);
	br = BPoint(bounds.right, bounds.bottom);
	
	v->SetHighColor(kSHADOW);
	v->StrokeLine(br, tr);
	v->StrokeLine(br, bl);
		
	v->SetHighColor(kHILITE);
	v->StrokeLine(tl, tr);
	v->StrokeLine(tl, bl);
	/***/
	
	v->Sync();
	bmap->Unlock();
	
	owner->Window()->Lock();
	owner->DrawBitmapAsync(bmap);
	owner->Window()->Unlock();
}

/*
Called when the owning view received a BMessage.
*/
void module_message_received(BMessage *msg)
{
	// nothing
}

/*
Called when the owning view receives a mouse click
*/
void module_mouse_down(BPoint, long)
{
	// silly
	beep();
}

/*
Called when the owning view receives a MouseMoved message.
*/
void module_mouse_moved(BPoint point, ulong transit, BMessage *msg)
{
	// nothing
}