/**************************************************************
 * perf_module.cpp
 * written by Steve Sprang
 * Copyright  1997
 *
 *************************************************************/
 
#include "module.h"

/* undocumented kernel routine */
extern "C" int  _kget_cpu_state_(int cpu);

// useful global variables
BView		*owner;
BRect		bounds;
BBitmap		*bmap;
BView		*v;
double		*oldActiveTime, *CPUTime, oldSystemTime;
		
/**************************
 * module_get_width()
 *************************/
long module_get_width()
{
	system_info		sysInfo;
	get_system_info(&sysInfo);
	
	// want enough space for all the processors and RAM
	return((sysInfo.cpu_count+1) * 8 + (sysInfo.cpu_count+2) * 4) + 1;
}

/**************************
 * module_create()
 *************************/
void module_create(BView *mod)
{
	owner = mod; // store owning view
	bounds = owner->Bounds(); // store bounds for quick access
	
	// create a new bitmap for drawing
	bmap = new BBitmap(bounds, B_COLOR_8_BIT, TRUE);
	v = new BView(bounds, "", B_FOLLOW_NONE, B_WILL_DRAW);
	bmap->AddChild(v);
	
	// init data structures needed for calculating CPULoad
	system_info		sysInfo;
	get_system_info(&sysInfo);
	oldActiveTime = new double[sysInfo.cpu_count]; 
	CPUTime = new double[sysInfo.cpu_count]; 
	oldSystemTime = system_time();
	
	for(int i = 0; i < sysInfo.cpu_count; i++) {
		oldActiveTime[i] = sysInfo.cpu_infos[i].active_time;
		CPUTime[i] = 0.0; // important!
	}
}

/**************************
 * module_quit()
 *************************/
void module_quit()
{
	// free up space
	delete bmap;
	delete oldActiveTime;
	delete CPUTime;
}

/**************************
 * module_pulse()
 *************************/
void module_pulse()
{
	system_info		sysInfo;
	double			now;
	
	now  = system_time();
	get_system_info(&sysInfo);
		
	// calculate the CPU load fraction for each CPU
	for(int i = 0; i < sysInfo.cpu_count; i++) {
		CPUTime[i] = (sysInfo.cpu_infos[i].active_time - oldActiveTime[i]) 
			/ (now - oldSystemTime);
			
		// normalize time
		if(CPUTime[i] > 1.0) CPUTime[i] = 1.0;
		
		// store values for next iteration
		oldActiveTime[i] = sysInfo.cpu_infos[i].active_time;
	}
	oldSystemTime = now;
	
	// update
	module_draw();
}

/**************************
 * module_draw()
 *************************/
void module_draw()
{
	BRect	meter; 
	BPoint	left, right;
	
	// create meter rectangle
	meter.Set(4,4,12,26);
	
	system_info		sysInfo;
	get_system_info(&sysInfo);
	
	// draw cool 3D background
	bmap->Lock();
	v->SetHighColor(kBGCOLOR);
	v->FillRect(bounds);
	
	BPoint tl, tr, bl, br;
	tl = BPoint(bounds.left, bounds.top);
	tr = BPoint(bounds.right, bounds.top);
	bl = BPoint(bounds.left, bounds.bottom);
	br = BPoint(bounds.right, bounds.bottom);
	
	v->SetHighColor(kSHADOW);
	v->StrokeLine(br, tr);
	v->StrokeLine(br, bl);
		
	v->SetHighColor(kHILITE);
	v->StrokeLine(tl, tr);
	v->StrokeLine(tl, bl);
	// end draw cool 3d background
	
	/* CPUs */
	// for each CPU draw a meter
	for(int i = 0; i < sysInfo.cpu_count; i++) {
		// init endpoints for line segment
		left.Set(meter.left + 1, meter.bottom -1);
		right.Set(meter.right -1, meter.bottom -1);
		
		// fill in the meter
		v->SetHighColor(0,40,0);
		v->FillRect(meter);
		
		if(_kget_cpu_state_(i)) {
			v->SetHighColor(0,255,0);
			// if the processor is active draw enough segments to indicate activity
			for(int n = 0; n < CPUTime[i] * 11; n++) {
				v->StrokeLine(left, right);
				left.y -= 2;
				right.y -= 2;
			}
		}
		else {
			// if inactive draw one red segment
			v->SetHighColor(255,0,0);
			v->StrokeLine(left, right);
		}
		// scoot down for next meter
		meter.OffsetBy(12,0);
	}
	/****/
	
	/* RAM */
	// init endpoints for line segment
	left.Set(meter.left + 1, meter.bottom -1);
	right.Set(meter.right -1, meter.bottom -1);
		
	v->SetHighColor(0,0,40);
	v->FillRect(meter);
	v->SetHighColor(0,200,255);
	// fill in enough segments to show RAM utilization
	for(int n = 0; n < ((double)sysInfo.used_pages / sysInfo.max_pages) * 11; n++) {
		v->StrokeLine(left, right);
		left.y -= 2;
		right.y -= 2;
	}
	/****/
	
	v->Sync(); // don't forget this, I usually do! :)
	bmap->Unlock();
	
	// draw the bitmap
	owner->Window()->Lock();
	owner->DrawBitmapAsync(bmap);
	owner->Window()->Unlock();
}

/**************************
 * module_message_received()
 *************************/
void module_message_received(BMessage *msg)
{
	// nothing
}

/**************************
 * module_mouse_down()
 *************************/
void module_mouse_down(BPoint, long)
{	
	// launch PerfMonitor, another of my little utilities
	be_roster->Launch('pmon');
}

/**************************
 * module_get_width()
 *************************/
void module_mouse_moved(BPoint point, ulong transit, BMessage *msg)
{
	// nothing
}

	
	